var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import React, { Fragment, useContext, useEffect, useMemo, useReducer, useState, } from 'react';
import AuthContext from './AuthContext';
import ConfigContext from './ConfigContext';
import * as orderingCore from '@orderingstack/ordering-core';
import axios from 'axios';
import { getErrorMessage, hasValidAuthorities } from './utils';
import { ModuleContext } from './ModuleContext';
import 'abortcontroller-polyfill/dist/abortcontroller-polyfill-only';
import { Mutex } from './utils/mutex';
import { DefaultDeviceCode } from './components/DefaultDeviceCode';
import { jwtDecode } from 'jwt-decode';
const mutex = new Mutex(0);
const _refreshStorageHandler = {
    getRefreshToken: (tenant) => {
        return (window.localStorage.getItem(`orderingstack_${tenant}_refreshToken`) || '');
    },
    setRefreshToken: (tenant, token) => {
        window.localStorage.setItem(`orderingstack_${tenant}_refreshToken`, token);
    },
    clearRefreshToken: (tenant) => {
        window.localStorage.removeItem(`orderingstack_${tenant}_refreshToken`);
    },
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function isSuccess(data) {
    return !(data === null || data === void 0 ? void 0 : data.error) && !!(data === null || data === void 0 ? void 0 : data.refresh_token);
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unused-vars
function isPendingAuth(data) {
    return (data === null || data === void 0 ? void 0 : data.error) === 'authorization_pending';
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unused-vars
function isSlowDown(data) {
    return (data === null || data === void 0 ? void 0 : data.error) === 'slow_down';
}
function deviceLoginReducer(state, action) {
    const { type, payload } = action;
    switch (type) {
        case 'enable':
            return Object.assign(Object.assign({}, state), { isDeviceCode: true });
        case 'loading':
            return Object.assign(Object.assign({}, state), { isDeviceCode: true, loading: true });
        case 'setValues':
            return Object.assign(Object.assign({}, state), { data: payload, error: undefined, loading: false });
        case 'error':
            return Object.assign(Object.assign({}, state), { error: payload });
        case 'disable':
            return {
                tenant: state.tenant,
                isDeviceCode: false,
                baseUrl: state.baseUrl,
                user: state.user,
            };
        default:
            return state;
    }
}
export function checkDisallowedAuthorities({ 
// roles or authorities
authorities, disallowedAuthorities, }) {
    if (!disallowedAuthorities ||
        disallowedAuthorities.length === 0 ||
        !authorities) {
        return [];
    }
    return authorities.filter((authority) => disallowedAuthorities.some((a) => {
        if (typeof a === 'string') {
            return a === authority;
        }
        if (a instanceof RegExp) {
            return a.test(authority);
        }
        return false;
    }));
}
export default function AuthWrapper(props) {
    const refreshStorageHandler = props.refreshStorageHandler || _refreshStorageHandler;
    const _configContext = useContext(ConfigContext);
    if (!_configContext) {
        return React.createElement(Fragment, null, "Configuration should be provided");
    }
    const config = _configContext;
    const { baseUrl, tenant } = config;
    const api = useMemo(() => axios.create({ baseURL: baseUrl }), [baseUrl]);
    const [auth, setAuth] = useState({
        loggedIn: false,
        UUID: '',
        authProvider: () => Promise.resolve({ token: '', UUID: '' }),
        signOut: onSignOut,
    });
    const [moduleConfig, setModuleConfig] = useState(undefined);
    const [resolved, setResolved] = useState(false);
    const [state, dispatch] = useReducer(deviceLoginReducer, {
        isDeviceCode: false,
        tenant: tenant,
        baseUrl: baseUrl,
    });
    const [disallowedRoles, setDisallowedRoles] = useState(null);
    function onSignOut() {
        console.log('--- log out ----');
        setAuth({
            loggedIn: false,
            UUID: '',
            authProvider: () => Promise.resolve({ token: '', UUID: '' }),
            signOut: onSignOut,
        });
        setModuleConfig(undefined);
        setResolved(true);
        orderingCore.clearAuthData(config.tenant, refreshStorageHandler);
    }
    function getAuthoritiesAndScopes(accessToken) {
        try {
            const { authorities, scope } = jwtDecode(accessToken);
            return { authorities, scope };
        }
        catch (e) {
            console.warn('error decoding access token', e);
        }
        return {
            authorities: [],
            scope: [],
        };
    }
    function getUserData(baseUrl, authProvider, setAuth) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a;
            let haveToken = false;
            try {
                const { UUID, token } = yield authProvider();
                if (token) {
                    const userData = yield orderingCore.getLoggedUserData(baseUrl, token);
                    const { authorities } = getAuthoritiesAndScopes(token);
                    // TODO handle roles checking differently for token with scopes
                    if (!hasValidAuthorities({
                        acceptedAuthorities: props.authorities,
                        authorities,
                    })) {
                        console.warn('--- USER HAS NO REQUIRED ROLES ----- ');
                        onSignOut();
                        setResolved(true);
                        alert('User has no required roles');
                        return false;
                    }
                    const userDisallowedRoles = checkDisallowedAuthorities({
                        authorities,
                        disallowedAuthorities: props.disallowedAuthorities,
                    });
                    if (userDisallowedRoles.length) {
                        console.warn('--- USER HAS DISALLOWED ROLES ----- ', userDisallowedRoles);
                        (_a = props.appInsights) === null || _a === void 0 ? void 0 : _a.trackTrace({
                            message: 'Front user has disallowed roles',
                            severityLevel: 2, // Warning
                            properties: {
                                uuid: UUID,
                                roles: userData.roles,
                                userDisallowedRoles,
                            },
                        });
                        if (props.logoutOnDisallowedRoles) {
                            setDisallowedRoles(userDisallowedRoles);
                            onSignOut();
                            setResolved(true);
                            return false;
                        }
                    }
                    // module config is automatically fetched by the ordering-core on each accessToken refresh
                    // const module = await orderingCore.getModuleConfig(
                    //   config.baseUrl,
                    //   token,
                    // );
                    // if (module) {
                    //   setModuleConfig(module);
                    // }
                    setAuth({
                        loggedIn: true,
                        email: userData.login,
                        UUID: UUID,
                        authProvider: authProvider,
                        signOut: onSignOut,
                    });
                    haveToken = true;
                }
                // eslint-disable-next-line @typescript-eslint/no-unused-vars
            }
            catch (err) {
                onSignOut();
            }
            setResolved(true);
            return haveToken;
        });
    }
    function deviceCodeLogin(moduleId, abortController) {
        return __awaiter(this, void 0, void 0, function* () {
            dispatch({ type: 'enable' });
            while (!abortController.signal.aborted) {
                try {
                    dispatch({ type: 'loading' });
                    // Get code
                    const { data: resData } = yield api.post('/auth-oauth2/oauth/device', { module: moduleId }, {
                        signal: abortController.signal,
                        headers: {
                            Accept: 'application/json',
                            'X-Tenant': tenant,
                            'Content-Type': 'application/x-www-form-urlencoded',
                            Authorization: `Basic ${config.basicAuth}`,
                        },
                    });
                    const exp = Date.now() + resData.expires_in * 1000;
                    dispatch({ type: 'setValues', payload: Object.assign(Object.assign({}, resData), { exp }) });
                    let success = false;
                    while (!abortController.signal.aborted && Date.now() < exp - 2000) {
                        const { data: tokenData } = yield api.post('/auth-oauth2/oauth/token', {
                            device_code: resData.device_code,
                            grant_type: 'urn:ietf:params:oauth:grant-type:device_code',
                        }, {
                            signal: abortController.signal,
                            headers: {
                                Accept: 'application/json',
                                'X-Tenant': tenant,
                                'Content-Type': 'application/x-www-form-urlencoded',
                                Authorization: `Basic ${config.basicAuth}`,
                            },
                            validateStatus: (status) => status < 500,
                        });
                        if (!isSuccess(tokenData)) {
                            yield new Promise((resolve) => setTimeout(resolve, resData.interval * 1000));
                        }
                        else {
                            yield refreshStorageHandler.setRefreshToken(tenant, tokenData.refresh_token);
                            // @ts-expect-error outdated type
                            orderingCore.setAuthData(tenant, tokenData, refreshStorageHandler);
                            success = true;
                            break;
                        }
                    }
                    if (success) {
                        break;
                    }
                }
                catch (e) {
                    if (e instanceof Error && e.name === 'AbortError') {
                        break;
                    }
                    const err = getErrorMessage(e);
                    console.log('err', err);
                    // TODO check if kind is INVALID_MODULE
                    if (err.message) {
                        dispatch({ type: 'error', payload: err });
                    }
                    yield new Promise((resolve) => setTimeout(resolve, 10000));
                    console.warn(e);
                    // continue loop
                }
            }
            dispatch({ type: 'disable' });
        });
    }
    useEffect(() => {
        if (auth.loggedIn) {
            return;
        }
        const abortController = new AbortController();
        function init(abortCtrl) {
            return __awaiter(this, void 0, void 0, function* () {
                //console.log("AutWrapper init --- ");
                const authProvider = orderingCore.createAuthDataProvider({
                    BASE_URL: config.baseUrl,
                    TENANT: config.tenant,
                    BASIC_AUTH: config.basicAuth,
                    anonymousAuth: config.anonymousOnInit,
                }, refreshStorageHandler, false, 
                // eslint-disable-next-line @typescript-eslint/no-unused-vars
                (error) => {
                    setAuth((prev) => (Object.assign(Object.assign({}, prev), { loggedIn: false, UUID: '' })));
                    setResolved(false);
                }, (modConfig) => setModuleConfig(modConfig));
                const params = new URLSearchParams(window.location.search);
                const moduleId = params.get('module') || config.moduleId;
                const refreshToken = params.get('refreshToken') || config.refreshToken;
                const { token } = yield authProvider();
                console.log('moduleId', moduleId, 'token exists?', !!token);
                if (moduleId && !token && !refreshToken) {
                    yield mutex.getLock(() => deviceCodeLogin(moduleId, abortCtrl));
                }
                // Check for refreshToken in search params
                try {
                    if (refreshToken) {
                        refreshStorageHandler.setRefreshToken(config.tenant, refreshToken);
                        // Remove refresh token from search params
                        params.delete('refreshToken');
                        const url = `${window.location.origin}${window.location.pathname}?${params.toString()}`;
                        window.history.replaceState(null, '', url);
                    }
                }
                catch (e) {
                    console.warn(e);
                }
                void getUserData(config.baseUrl, authProvider, setAuth);
                window.addEventListener('message', function (event) {
                    if (event.origin !== config.baseUrl)
                        return;
                    const authData = event.data;
                    // console.log(authData);
                    if (authData.UUID === '') {
                        setAuth({
                            loggedIn: false,
                            UUID: '',
                            authProvider: authProvider,
                            signOut: onSignOut,
                        });
                    }
                    else {
                        orderingCore.setAuthData(config.tenant, {
                            UUID: authData.UUID,
                            access_token: authData.access_token,
                            refresh_token: authData.refresh_token,
                            expires_in: authData.expires_in,
                        }, refreshStorageHandler);
                        setAuth({
                            loggedIn: true,
                            UUID: authData.UUID,
                            authProvider: authProvider,
                            signOut: onSignOut,
                        });
                        getUserData(config.baseUrl, authProvider, setAuth);
                    }
                });
            });
        }
        void init(abortController);
        return () => {
            abortController.abort();
        };
    }, [auth === null || auth === void 0 ? void 0 : auth.loggedIn, config]);
    if (state.isDeviceCode) {
        const DeviceCodeComp = props.DeviceCodeComp || DefaultDeviceCode;
        return React.createElement(DeviceCodeComp, Object.assign({}, state));
    }
    if (disallowedRoles === null || disallowedRoles === void 0 ? void 0 : disallowedRoles.length) {
        const DisallowedRolesComp = props.DisallowedRolesComp || DefaultDisallowedRolesComp;
        return React.createElement(DisallowedRolesComp, null);
    }
    return (React.createElement(Fragment, null,
        React.createElement(ModuleContext.Provider, { value: moduleConfig },
            React.createElement(AuthContext.Provider, { value: Object.assign(Object.assign({}, auth), { resolved }) }, props.children))));
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export function ShowWhenAuthenticated(props) {
    const { loggedIn } = useContext(AuthContext);
    return loggedIn ? React.createElement(Fragment, null, props.children) : null;
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export function ShowWhenNotAuthenticated(props) {
    const { loggedIn } = useContext(AuthContext);
    return !loggedIn ? React.createElement(Fragment, null, props.children) : null;
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export function ShowWhenNotAuthenticatedAnResolved(props) {
    const { loggedIn, resolved } = useContext(AuthContext);
    return !loggedIn && resolved ? React.createElement(Fragment, null, props.children) : null;
}
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export function ShowWhenAuthenticating(props) {
    const { resolved } = useContext(AuthContext);
    return !resolved ? React.createElement(Fragment, null, props.children) : null;
}
function DefaultDisallowedRolesComp() {
    return (React.createElement("div", { style: {
            width: '100%',
            height: '100vh',
            display: 'flex',
            flexDirection: 'column',
            alignItems: 'center',
            justifyContent: 'center',
            fontSize: '2vw',
            backgroundColor: 'white',
        } },
        React.createElement("div", { style: {
                textAlign: 'center',
                display: 'flex',
                flexDirection: 'column',
                alignItems: 'center',
            } },
            React.createElement("h1", null, "Auth error 101. Contact support"))));
}
